c
c This file contains routines that substitute the helas calls for the triple-vector-boson
c couplings with anomalous couplings.
c

      subroutine wwa_anomal6(wm, wp, a, vertex)
c
c This subroutine computes an amplitude of the three-point coupling of
c the gauge bosons w+ w- a with anomalous couplings.
c
c input:
c       complex wm(6)          : vector               flow-out W-
c       complex wp(6)          : vector               flow-out W+
c       complex a(6)           : vector               flow-out A
c output:
c       complex vertex         : amplitude               gamma(wm,wp,a)
c     
      implicit none
      double complex wm(6),wp(6),a(6),vertex, dotcc, dotrc
      double precision pwm(0:3),pwp(0:3),pa(0:3),dotrr
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      pwm(0) = dble( wm(5))
      pwm(1) = dble( wm(6))
      pwm(2) = dimag(wm(6))
      pwm(3) = dimag(wm(5))
      pwp(0) = dble( wp(5))
      pwp(1) = dble( wp(6))
      pwp(2) = dimag(wp(6))
      pwp(3) = dimag(wp(5))
      pa(0) = dble( a(5))
      pa(1) = dble( a(6))
      pa(2) = dimag(a(6))
      pa(3) = dimag(a(5))

      vertex = - ( dotcc(wm,wp)*(dotrc(pwm,a )-dotrc(pwp,a ))
     &           + dotcc(wp,a )*(dotrc(pwp,wm)-dotrc(pa ,wm))
     &           + dotcc(a ,wm)*(dotrc(pa ,wp)-dotrc(pwm,wp))  )*gwwa
      
      if (fwww .ne. 0) then
        vertex = vertex + fwww * 3./2. * gw**3 * sqrt(sin2w) * ( dotrc(pa,wm) * dotrc(pwp,a) * dotrc(pwm,wp)
     &      - dotrc(pa,wp) * dotrc(pwp,wm) * dotrc(pwm,a)
     &      + dotrr(pa,pwm) * ( dotrc(pwp,wm) * dotcc(wp,a) - dotrc(pwp,a) * dotcc(wp,wm) )
     &      + dotrr(pwm,pwp) * ( dotrc(pa,wp) * dotcc(wm,a) - dotrc(pa,wm) * dotcc(wp,a) )
     &      + dotrr(pwp,pa) * ( dotrc(pwm,a) * dotcc(wp,wm) - dotrc(pwm,wp) * dotcc(wm,a) ) )
      endif

      if (fw .ne. 0) then
        vertex = vertex + fw * gw * wmass**2 / 2. * sqrt(sin2w) * ( dotrc(pa,wm)*dotcc(wp,a) - dotrc(pa,wp)*dotcc(wm,a) )
      endif

      if (fb .ne. 0) then
        vertex = vertex + fb * gw * wmass**2 / 2. * sqrt(sin2w) * ( dotrc(pa,wm)*dotcc(wp,a) - dotrc(pa,wp)*dotcc(wm,a) )
      endif

      return
      end


      subroutine wwz_anomal6(wm, wp, z, vertex)
c
c This subroutine computes an amplitude of the three-point coupling of
c the gauge bosons w+ w- z with anomalous couplings.
c
c input:
c       complex wm(6)          : vector               flow-out W-
c       complex wp(6)          : vector               flow-out W+
c       complex z(6)           : vector               flow-out Z
c output:
c       complex vertex         : amplitude               gamma(wm,wp,z)
c     
      implicit none
      double complex wm(6),wp(6),z(6),vertex, dotcc, dotrc
      double precision pwm(0:3),pwp(0:3),pz(0:3), dotrr, cos2w
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      cos2w=1.0d0 - sin2w

      pwm(0) = dble( wm(5))
      pwm(1) = dble( wm(6))
      pwm(2) = dimag(wm(6))
      pwm(3) = dimag(wm(5))
      pwp(0) = dble( wp(5))
      pwp(1) = dble( wp(6))
      pwp(2) = dimag(wp(6))
      pwp(3) = dimag(wp(5))
      pz(0) = dble( z(5))
      pz(1) = dble( z(6))
      pz(2) = dimag(z(6))
      pz(3) = dimag(z(5))

      vertex = - ( dotcc(wm,wp)*(dotrc(pwm,z )-dotrc(pwp,z ))
     &           + dotcc(wp,z )*(dotrc(pwp,wm)-dotrc(pz ,wm))
     &           + dotcc(z ,wm)*(dotrc(pz ,wp)-dotrc(pwm,wp))  )*gwwz

      if (fwww .ne. 0) then
        vertex = vertex + fwww * 3./2. * gw**3 * sqrt(cos2w) * ( dotrc(pz,wm) * dotrc(pwp,z) * dotrc(pwm,wp)
     &      - dotrc(pz,wp) * dotrc(pwp,wm) * dotrc(pwm,z)
     &      + dotrr(pz,pwm) * ( dotrc(pwp,wm) * dotcc(wp,z) - dotrc(pwp,z) * dotcc(wp,wm) )
     &      + dotrr(pwm,pwp) * ( dotrc(pz,wp) * dotcc(wm,z) - dotrc(pz,wm) * dotcc(wp,z) )
     &      + dotrr(pwp,pz) * ( dotrc(pwm,z) * dotcc(wp,wm) - dotrc(pwm,wp) * dotcc(wm,z) ) )
      endif

      if (fw .ne. 0) then
        vertex = vertex + fw * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * ( (dotrc(pwm,wp)-dotrc(pz,wp)) * dotcc(wm,z)
     &      + (dotrc(pz,wm)-dotrc(pwp,wm)) * dotcc(wp,z) + (dotrc(pwp,z)-dotrc(pwm,z)) * dotcc(wp,wm)
     &      - sin2w * ( dotrc(pz,wm)*dotcc(wp,z) - dotrc(pz,wp)*dotcc(wm,z) ) )
      endif

      if (fb .ne. 0) then
        vertex = vertex - fb * gw * wmass**2 / ( 2. * sqrt(cos2w) ) *  
     &       sin2w * ( dotrc(pz,wm)*dotcc(wp,z) - dotrc(pz,wp)*dotcc(wm,z) ) 
      endif

      return
      end



      subroutine jwpawm_anomal6(a, wm, jvv)
c
c This subroutine computes an off-shell vector current attached to a W+
c from the three-point gauge boson coupling w+ w- a with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex a(6)           : first  vector                        v1
c       complex wm(6)          : second vector                        wm
c output:
c       complex jvv(6)         : vector current            j^mu(v:a,wm)
c     
      implicit none
      double complex a(6), wm(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pa(4),pwm(4),q(4),s
      double precision vm2, dotrr
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   
 
      jvv(5) = a(5)+wm(5)
      jvv(6) = a(6)+wm(6)

      pa(1) =  dble( a(5))
      pa(2) =  dble( a(6))
      pa(3) =  dimag(a(6))
      pa(4) =  dimag(a(5))
      pwm(1) =  dble( wm(5))
      pwm(2) =  dble( wm(6))
      pwm(3) =  dimag(wm(6))
      pwm(4) =  dimag(wm(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)
      vm2 = wmass**2

c     Vertex
      do i=1,4
            j12(i) = -( (pa(i)-pwm(i))*dotcc(a,wm) 
     &                      + (dotrc(pwm,a)-dotrc(q,a))*wm(i) 
     &                      + (dotrc(q,wm)-dotrc(pa,wm))*a(i) )* gwwa

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(sin2w) * 
     &        ( dotrc(pa,wm) * dotrc(q,a) * pwm(i) - pa(i) * dotrc(q,wm) * dotrc(pwm,a)
     &        + dotrr(pa,pwm) * ( dotrc(q,wm) * a(i) - dotrc(q,a) * wm(i) )
     &        + dotrr(pwm,q) * ( pa(i) * dotcc(wm,a) - dotrc(pa,wm) * a(i) )
     &        + dotrr(q,pa) * ( dotrc(pwm,a) * wm(i) - pwm(i) * dotcc(wm,a) ) )
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &                           ( dotrc(pa,wm)*a(i) - pa(i)*dotcc(wm,a) )
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) + fb * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &                           ( dotrc(pa,wm)*a(i) - pa(i)*dotcc(wm,a) )
        endif

!             write(*,*) "XXXblubb1",i, j12(i)
!             write(*,*) "XXXa",i, a(i)
!               write(*,*) "XXXpa",i, pa(i)
      enddo


c     Propagator
      cm2=dcmplx( vm2, -wmass*wwidth )       !Fabio's implementation of the fixed width
      dg = 1.d0 / dcmplx( s-vm2, wmass*wwidth )       !factor of i^2 here
      do i=1,4
         jvv(i) = dg * ( j12(i) - q(i)*dotrc(q,j12)/cm2 )
      enddo

      return
      end



      subroutine jwpzwm_anomal6(z, wm, jvv)
c
c This subroutine computes an off-shell vector current attached to a W+
c from the three-point gauge boson coupling w+ w- z with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex z(6)           : first  vector                        z
c       complex wm(6)          : second vector                        wm
c output:
c       complex jvv(6)         : vector current            j^mu(v:z,wm)
c     
      implicit none
      double complex z(6), wm(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pz(4),pwm(4),q(4),s
      double precision vm2, dotrr, cos2w
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   
 
      cos2w=1.0d0 - sin2w

      jvv(5) = z(5)+wm(5)
      jvv(6) = z(6)+wm(6)

      pz(1) =  dble( z(5))
      pz(2) =  dble( z(6))
      pz(3) =  dimag(z(6))
      pz(4) =  dimag(z(5))
      pwm(1) =  dble( wm(5))
      pwm(2) =  dble( wm(6))
      pwm(3) =  dimag(wm(6))
      pwm(4) =  dimag(wm(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)
      vm2 = wmass**2

c     Vertex
      do i=1,4
            j12(i) = -gwwz * ( (pz(i)-pwm(i))*dotcc(z,wm) 
     &                                     + (dotrc(pwm,z)-dotrc(q,z))*wm(i) 
     &                                     + (dotrc(q,wm)-dotrc(pz,wm))*z(i) )

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(cos2w) * 
     &        ( dotrc(pz,wm) * dotrc(q,z) * pwm(i) - pz(i) * dotrc(q,wm) * dotrc(pwm,z)
     &        + dotrr(pz,pwm) * ( dotrc(q,wm) * z(i) - dotrc(q,z) * wm(i) )
     &        + dotrr(pwm,q) * ( pz(i) * dotcc(wm,z) - dotrc(pz,wm) * z(i) )
     &        + dotrr(q,pz) * ( dotrc(pwm,z) * wm(i) - pwm(i) * dotcc(wm,z) ) )
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * 
     &        ( (pwm(i)-pz(i)) * dotcc(wm,z)
     &        + (dotrc(pz,wm)-dotrc(q,wm)) * z(i) + (dotrc(q,z)-dotrc(pwm,z)) * wm(i)
     &        - sin2w * ( dotrc(pz,wm)*z(i) - pz(i)*dotcc(wm,z) ) )
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) - fb * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * 
     &        sin2w * ( dotrc(pz,wm)*z(i) - pz(i)*dotcc(wm,z) )
        endif

      enddo


c     Propagator
      cm2=dcmplx( vm2, -wmass*wwidth )       !Fabio's implementation of the fixed width
      dg = 1.d0 / dcmplx( s-vm2, wmass*wwidth )
      do i=1,4
         jvv(i) = dg * ( j12(i) - q(i)*dotrc(q,j12)/cm2 )
      enddo

      return
      end



      subroutine jwmwpa_anomal6(wp, a, jvv)
c
c This subroutine computes an off-shell vector current attached to a W-
c from the three-point gauge boson coupling w+ w- a with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex wp(6)          : first  vector                        wp
c       complex a(6)           : second vector                        a
c output:
c       complex jvv(6)         : vector current            j^mu(v:wp,a)
c     
      implicit none
      double complex wp(6), a(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pwp(4),pa(4),q(4),s
      double precision vm2, dotrr
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   
 
      jvv(5) = wp(5)+a(5)
      jvv(6) = wp(6)+a(6)

      pwp(1) =  dble( wp(5))
      pwp(2) =  dble( wp(6))
      pwp(3) =  dimag(wp(6))
      pwp(4) =  dimag(wp(5))
      pa(1) =  dble( a(5))
      pa(2) =  dble( a(6))
      pa(3) =  dimag(a(6))
      pa(4) =  dimag(a(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)
      vm2 = wmass**2

c     Vertex
      do i=1,4
            j12(i) = -gwwa * ( (pwp(i)-pa(i))*dotcc(wp,a) 
     &                                     + (dotrc(pa,wp)-dotrc(q,wp))*a(i) 
     &                                     + (dotrc(q,a)-dotrc(pwp,a))*wp(i) )

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(sin2w) *
     &        ( pa(i) * dotrc(pwp,a) * dotrc(q,wp) - dotrc(pa,wp) * pwp(i) * dotrc(q,a)
     &        + dotrr(pa,q) * ( pwp(i) * dotcc(wp,a) - dotrc(pwp,a) * wp(i) )
     &        + dotrr(q,pwp) * ( dotrc(pa,wp) * a(i) - pa(i) * dotcc(wp,a) )
     &        + dotrr(pwp,pa) * ( dotrc(q,a) * wp(i) - dotrc(q,wp) * a(i) ) )
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &        ( pa(i)*dotcc(wp,a) - dotrc(pa,wp)*a(i) )
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) + fb * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &        ( pa(i)*dotcc(wp,a) - dotrc(pa,wp)*a(i) )
        endif

      enddo


c     Propagator
      cm2=dcmplx( vm2, -wmass*wwidth )       !Fabio's implementation of the fixed width
      dg = 1.d0 / dcmplx( s-vm2, wmass*wwidth )
      do i=1,4
         jvv(i) = dg * ( j12(i) - q(i)*dotrc(q,j12)/cm2 )
      enddo

      return
      end



      subroutine jwmwpz_anomal6(wp, z, jvv)
c
c This subroutine computes an off-shell vector current attached to a W-
c from the three-point gauge boson coupling w+ w- z with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex wp(6)          : first  vector                        wp
c       complex z(6)           : second vector                        z
c output:
c       complex jvv(6)         : vector current wm         j^mu(v:wp,z)
c     
      implicit none
      double complex wp(6), z(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pwp(4),pz(4),q(4),s
      double precision vm2, dotrr, cos2w
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   
 
      cos2w=1.0d0 - sin2w

      jvv(5) = wp(5)+z(5)
      jvv(6) = wp(6)+z(6)

      pwp(1) =  dble( wp(5))
      pwp(2) =  dble( wp(6))
      pwp(3) =  dimag(wp(6))
      pwp(4) =  dimag(wp(5))
      pz(1) =  dble( z(5))
      pz(2) =  dble( z(6))
      pz(3) =  dimag(z(6))
      pz(4) =  dimag(z(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)
      vm2 = wmass**2

c     Vertex
      do i=1,4
            j12(i) = -gwwz * ( (pwp(i)-pz(i))*dotcc(wp,z) 
     &                                     + (dotrc(pz,wp)-dotrc(q,wp))*z(i) 
     &                                     + (dotrc(q,z)-dotrc(pwp,z))*wp(i) )

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(cos2w) * 
     &        ( pz(i) * dotrc(pwp,z) * dotrc(q,wp) - dotrc(pz,wp) * pwp(i) * dotrc(q,z)
     &        + dotrr(pz,q) * ( pwp(i) * dotcc(wp,z) - dotrc(pwp,z) * wp(i) )
     &        + dotrr(q,pwp) * ( dotrc(pz,wp) * z(i) - pz(i) * dotcc(wp,z) )
     &        + dotrr(pwp,pz) * ( dotrc(q,z) * wp(i) - dotrc(q,wp) * z(i) ) )
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * 
     &        ( (dotrc(q,wp)-dotrc(pz,wp)) * z(i)
     &        + (pz(i)-pwp(i)) * dotcc(wp,z) + (dotrc(pwp,z)-dotrc(q,z)) * wp(i)
     &        - sin2w * ( pz(i)*dotcc(wp,z) - dotrc(pz,wp)*z(i) ) )
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) - fb * gw * wmass**2 / ( 2. * sqrt(cos2w) ) *
     &        sin2w * ( pz(i)*dotcc(wp,z) - dotrc(pz,wp)*z(i) )
        endif

      enddo


c     Propagator
      cm2=dcmplx( vm2, -wmass*wwidth )       !Fabio's implementation of the fixed width
      dg = 1.d0 / dcmplx( s-vm2, wmass*wwidth )
      do i=1,4
         jvv(i) = dg * ( j12(i) - q(i)*dotrc(q,j12)/cm2 )
      enddo

      return
      end



      subroutine jaww_anomal6(wm, wp, jvv)
c
c This subroutine computes an off-shell vector current attached to an A
c from the three-point gauge boson coupling w+ w- a with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex wm(6)          : vector               flow-out W-
c       complex wp(6)          : vector               flow-out W+
c where all the bosons are defined by the flowing-OUT quantum number.
c
c output:
c       complex jvv(6)         : vector current A          j^mu(v:wp,wp)
c     
      implicit none
      double complex wm(6), wp(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pwm(4),pwp(4),q(4),s
      double precision vm2, dotrr
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      jvv(5) = wm(5)+wp(5)
      jvv(6) = wm(6)+wp(6)

      pwm(1) =  dble( wm(5))
      pwm(2) =  dble( wm(6))
      pwm(3) =  dimag(wm(6))
      pwm(4) =  dimag(wm(5))
      pwp(1) =  dble( wp(5))
      pwp(2) =  dble( wp(6))
      pwp(3) =  dimag(wp(6))
      pwp(4) =  dimag(wp(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)


c     Vertex
      do i=1,4
            j12(i) = -gwwa * (  dotcc(wm,wp)   *pwm(i)
     &                        - dotcc(wm,wp)   *pwp(i)
     &                        + dotrc(pwp,wm)  *wp(i)
     &                        - dotrc(q,wm)    *wp(i) 
     &                        + dotrc(q,wp)    *wm(i)
     &                        - dotrc(pwm,wp)  *wm(i) )

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(sin2w) * 
     &                       ( dotrc(q,wm)    * dotrc(pwm,wp) * pwp(i) 
     &                       - dotrr(q,pwm)   * dotcc(wm,wp)  * pwp(i)
     &                       + dotrr(pwp,q)   * dotcc(wm,wp)  * pwm(i)
     &                       - dotrc(q,wp)    * dotrc(pwp,wm) * pwm(i)
     &                       + dotrr(q,pwm)   * dotrc(pwp,wm) * wp(i) 
     &                       - dotrr(pwm,pwp) * dotrc(q,wm)   * wp(i)
     &                       + dotrr(pwm,pwp) * dotrc(q,wp)   * wm(i)
     &                       - dotrr(pwp,q)   * dotrc(pwm,wp) * wm(i)  )
 
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &        ( dotrc(q,wm)*wp(i) - dotrc(q,wp)*wm(i) )
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) + fb * gw * wmass**2 / 2. * sqrt(sin2w) * 
     &        ( dotrc(q,wm)*wp(i) - dotrc(q,wp)*wm(i) )
        endif

      enddo

c     Propagator
      dg = 1.d0 / s
      do i=1,4
         jvv(i) = dg*j12(i)
      enddo

      return
      end



      subroutine jzww_anomal6(wm, wp, jvv)
c
c This subroutine computes an off-shell vector current attached to a Z
c from the three-point gauge boson coupling w+ w- z with anomalous
c couplings.  The vector propagator is given in Feynman gauge for a 
c massless vector and in unitary gauge for a massive vector.
c
c input:
c       complex wm(6)          : vector               flow-out W-
c       complex wp(6)          : vector               flow-out W+
c where all the bosons are defined by the flowing-OUT quantum number.
c
c output:
c       complex jvv(6)         : vector current Z          j^mu(v:wm,wp)
c     
      implicit none
      double complex wm(6), wp(6), jvv(6), j12(4), dg, dotcc, dotrc
      double complex cm2        ! mass**2- I Gamma mass (Fabio)
      double precision pwm(4),pwp(4),q(4),s
      double precision vm2, dotrr, cos2w
      integer i
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   
 
      cos2w=1.0d0 - sin2w
 
      jvv(5) = wm(5)+wp(5)
      jvv(6) = wm(6)+wp(6)

      pwm(1) =  dble( wm(5))
      pwm(2) =  dble( wm(6))
      pwm(3) =  dimag(wm(6))
      pwm(4) =  dimag(wm(5))
      pwp(1) =  dble( wp(5))
      pwp(2) =  dble( wp(6))
      pwp(3) =  dimag(wp(6))
      pwp(4) =  dimag(wp(5))
      q(1)  = -dble( jvv(5))
      q(2)  = -dble( jvv(6))
      q(3)  = -dimag(jvv(6))
      q(4)  = -dimag(jvv(5))
      s = dotrr(q,q)
      vm2 = zmass**2

c     Vertex
      do i=1,4
            j12(i) = -gwwz * ( dotcc(wm,wp)* pwm(i) 
     &                       - dotcc(wm,wp) * pwp(i)
     &                       + dotrc(pwp,wm)* wp(i)
     &                       - dotrc(q,wm)  * wp(i) 
     &                       + dotrc(q,wp)  * wm(i)
     &                       - dotrc(pwm,wp)* wm(i) )

        if (fwww .ne. 0) then
          j12(i) = j12(i) + fwww * 3./2. * gw**3 * sqrt(cos2w) * 
     &        ( dotrc(q,wm)    * dotrc(pwm,wp) * pwp(i)
     &        - dotrr(q,pwm)   * dotcc(wm,wp)  * pwp(i) 
     &        + dotrr(pwp,q)   * dotcc(wm,wp)  * pwm(i)
     &        - dotrc(q,wp)    * dotrc(pwp,wm) * pwm(i)
     &        + dotrr(q,pwm)   * dotrc(pwp,wm) * wp(i) 
     &        - dotrr(pwm,pwp) * dotrc(q,wm)   * wp(i)
     &        + dotrr(pwm,pwp) * dotrc(q,wp)   * wm(i) 
     &        - dotrr(pwp,q)   * dotrc(pwm,wp) * wm(i)  )
        endif

        if (fw .ne. 0) then
          j12(i) = j12(i) + fw * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * 
     &        (  dotrc(pwm,wp)     * wm(i)       !g1
     &         - dotrc(q,wp)       * wm(i)       !deltakappa
     &         + dotrc(q,wm)       * wp(i)       !deltakappa
     &         - dotrc(pwp,wm)     * wp(i)       !g1 
     &         + dotcc(wm,wp)      *pwp(i)       !g1
     &         - dotcc(wm,wp)      *pwm(i)       !g1
     &         + sin2w*dotrc(q,wp) * wm(i)       !deltakappa
     &         - sin2w*dotrc(q,wm) * wp(i)       !deltakappa
     &          ) 
        endif

        if (fb .ne. 0) then
          j12(i) = j12(i) + fb * gw * wmass**2 / ( 2. * sqrt(cos2w) ) * 
     &        sin2w *      (   + dotrc(q,wp)*wm(i)   !deltakappa
     &                         - dotrc(q,wm)*wp(i))  !deltakappa
        endif

      enddo


c     Propagator
      cm2=dcmplx( vm2, -zmass*zwidth )       !Fabio's implementation of the fixed width
      dg = 1.d0 / dcmplx( s-vm2, zmass*zwidth )
      do i=1,4
         jvv(i) = dg * ( j12(i) - q(i)*dotrc(q,j12)/cm2 )
      enddo


      return
      end



      subroutine azh_anomal6(a, z, sc, vertex)
c
c This subroutine computes an amplitude of the vector-vector-scalar
c coupling a z h with anomalous couplings.
c
c input:
c       complex a(6)          : photon
c       complex z(6)          : z-boson
c       complex sc(3)          : higgs
c
c output:
c       complex vertex         : amplitude                gamma(a,z,s)
c     
      implicit none
      double complex a(6),z(6),sc(3),vertex, dotcc, dotrc
      double precision pa(4),pz(4),q(4), dotrr, cos2w
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      cos2w=1.0d0 - sin2w

      pa(1) = dble( a(5))
      pa(2) = dble( a(6))
      pa(3) = dimag(a(6))
      pa(4) = dimag(a(5))
      pz(1) = dble( z(5))
      pz(2) = dble( z(6))
      pz(3) = dimag(z(6))
      pz(4) = dimag(z(5))
      q(1)  = dble( sc(2))
      q(2)  = dble( sc(3))
      q(3)  = dimag(sc(3))
      q(4)  = dimag(sc(2))

      vertex = dcmplx(0.,0.)

      if (fw .ne. 0) then
         vertex = vertex + fw * gw * wmass * sqrt(sin2w/cos2w) / 2. * sc(1) * ( dotrr(pa,q) * dotcc(a,z)
     &      - dotrc(q,a)*dotrc(pa,z) )
      endif

      if (fb .ne. 0) then
         vertex = vertex + fb * gw * wmass * sqrt(sin2w/cos2w) / 2. * sc(1) * ( - dotrr(pa,q) * dotcc(a,z)
     &      + dotrc(q,a)*dotrc(pa,z) )
      endif

      return
      end



      subroutine zzh_anomal6(v1, v2, sc, vertex)
c
c This subroutine computes an amplitude of the vector-vector-scalar
c coupling z z h with anomalous couplings.
c
c input:
c       complex z1(6)          : first  vector                        z1
c       complex z2(6)          : second vector                        z2
c       complex sc(3)          : input  scalar                        s
c
c output:
c       complex vertex         : amplitude                gamma(z1,z2,s)
c     
      implicit none
      double complex v1(6),v2(6),sc(3),vertex, dotcc, dotrc
      double precision p1(4),p2(4),q(4), dotrr, cos2w
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      cos2w=1.0d0 - sin2w

      p1(1) = dble( v1(5))
      p1(2) = dble( v1(6))
      p1(3) = dimag(v1(6))
      p1(4) = dimag(v1(5))
      p2(1) = dble( v2(5))
      p2(2) = dble( v2(6))
      p2(3) = dimag(v2(6))
      p2(4) = dimag(v2(5))
      q(1)  = dble( sc(2))
      q(2)  = dble( sc(3))
      q(3)  = dimag(sc(3))
      q(4)  = dimag(sc(2))

      vertex = gw*zmass/sqrt(cos2w)*sc(1)*dotcc(v1,v2)

      if (fw .ne. 0) then
         vertex = vertex + fw * gw * wmass / 2. * sc(1) * ( (dotrr(q,p1) + dotrr(q,p2)) * dotcc(v1,v2)
     &      - dotrc(q,v1)*dotrc(p1,v2) - dotrc(q,v2)*dotrc(p2,v1) )
      endif

      if (fb .ne. 0) then
         vertex = vertex + fb * gw * sin2w / cos2w * wmass / 2. * sc(1) * ( (dotrr(q,p1) + dotrr(q,p2)) * dotcc(v1,v2)
     &      - dotrc(q,v1)*dotrc(p1,v2) - dotrc(q,v2)*dotrc(p2,v1) )
      endif

      return
      end



      subroutine wwh_anomal6(v1, v2, sc, vertex)
c
c This subroutine computes an amplitude of the vector-vector-scalar
c coupling w+ w- h with anomalous couplings.
c
c input:
c       complex wm(6)          : first  vector                        wm
c       complex wp(6)          : second vector                        wp
c       complex sc(3)          : input  scalar                        s
c
c output:
c       complex vertex         : amplitude                gamma(wm,wp,s)
c     
      implicit none
      double complex v1(6),v2(6),sc(3),vertex, dotcc, dotrc
      double precision p1(4),p2(4),q(4), dotrr, cos2w
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      cos2w=1.0d0 - sin2w

      p1(1) = dble( v1(5))
      p1(2) = dble( v1(6))
      p1(3) = dimag(v1(6))
      p1(4) = dimag(v1(5))
      p2(1) = dble( v2(5))
      p2(2) = dble( v2(6))
      p2(3) = dimag(v2(6))
      p2(4) = dimag(v2(5))
      q(1)  = dble( sc(2))
      q(2)  = dble( sc(3))
      q(3)  = dimag(sc(3))
      q(4)  = dimag(sc(2))

      vertex = gw*wmass*sc(1)*dotcc(v1,v2)

      if (fw .ne. 0) then
         vertex = vertex + fw * gw * wmass / 2. * sc(1) * ( (dotrr(q,p1) + dotrr(q,p2)) * dotcc(v1,v2)
     &      - dotrc(q,v1)*dotrc(p1,v2) - dotrc(q,v2)*dotrc(p2,v1) )
      endif

      return
      end



      subroutine jhww_anomal6(v1,v2 , hvv)
c
c This subroutine computes an off-shell scalar current from the vector-
c vector-scalar coupling w w h with anomalous couplings.
c
c input:
c       complex wm(6)          : first  vector                        wm
c       complex wp(6)          : second vector                        wp
c
c output:
c       complex hvv(3)         : off-shell scalar current     j(s:wm,wp)
c     
      implicit none
      double complex v1(6),v2(6),hvv(3),dg, dotrc, dotcc
      double precision p1(4),p2(4),q(4),q2, dotrr
      external dotcc, dotrc, dotrr

#include "VBFNLO/utilities/coupl.inc"
#include "VBFNLO/utilities/an_couplings.inc"   

      hvv(2) = v1(5)+v2(5)
      hvv(3) = v1(6)+v2(6)

      p1(1) = dble( v1(5))
      p1(2) = dble( v1(6))
      p1(3) = dimag(v1(6))
      p1(4) = dimag(v1(5))
      p2(1) = dble( v2(5))
      p2(2) = dble( v2(6))
      p2(3) = dimag(v2(6))
      p2(4) = dimag(v2(5))
      q(1) = -dble( hvv(2))
      q(2) = -dble( hvv(3))
      q(3) = -dimag(hvv(3))
      q(4) = -dimag(hvv(2))
      q2 = dotrr(q,q)

c     Vertex:
      hvv(1) = gw*wmass*dotcc(v1,v2)

      if (fw .ne. 0) then
         hvv(1) = hvv(1) + fw * gw * wmass / 2. * ( (dotrr(q,p1) + dotrr(q,p2)) * dotcc(v1,v2)
     &      - dotrc(q,v1)*dotrc(p1,v2) - dotrc(q,v2)*dotrc(p2,v1) )
      endif

c     Propagator:
      hvv(1) = - hvv(1) / dcmplx( q2-hmass**2, hmass*hwidth )

      return
      end
