 
****************************************************************************
*   This file contains functions and subroutines for the cuts.
c***************************************************************************
c   LIST OF ALL FUNCTIONS AND SUBROUTINES IN THIS FILE:
c
c     FUNCTION Cuts(jets, njets, v, L)
c     FUNCTION Basic_Cuts(jets, njets, v)
c     FUNCTION VBF_Cuts(jets, njets, v)
c     FUNCTION JetVeto_Cuts(jets, njets, v)
c     SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c
c***************************************************************************
*   Last modified: 02.02.2011
****************************************************************************
c*************************************************************************
      SUBROUTINE InitCuts
c*************************************************************************
c     Reads in the values for the cuts.
c*************************************************************************
      implicit none
#include "global.inc"
#include "cuts.inc"

      integer LSP
      common /LSP_def/ LSP

      call loadfile("cuts.dat",.false.)

c jet cuts
      
      call read_realList("PT_JET_MIN",4,ptmin_def,ptmin_def_default,.true.)
      call read_real("Y_P_MAX",ymax_part,5d0)
      call read_Real("PGENKTJET",jetgenktp,1d0) ! 1:kT, 0:C/A, -1:anti-kT 
      call read_real("Y_JET_MAX",ymax_def,4.5d0)
      call read_real("RJJ_MIN",rjjmin,0.8d0)

c leptons cuts
      call read_real("RLL_MIN",rll_min,0.4d0)
      call read_real("RLL_MAX",rll_max,50.0d0)
      call read_real("RJL_MIN",rjl_min,0.6d0)
      call read_real("Y_L_MAX", ymax_l,2.5d0)
      call read_real("PT_L_MIN", ptlmin, 10d0)
*      call read_real("PT_L_MAX", ptlmax, 14000d0)
      call read_real("MLL_MIN",mllmin,15d0)
      call read_real("MLL_MAX",mllmax,14000d0)
*      call read_real("PT_IN_MIN", ptinmin, 10d0)

c photon cuts
      call read_real("RGG_MIN",rgg_min,0.6d0)
      call read_real("RGG_MAX",rgg_max,50.0d0)
      call read_real("RJG_MIN",rjg_min,0.6d0)
      call read_real("RLG_MIN",rlg_min,0.6d0)
      call read_real("Y_G_MAX",ymax_g,1.5d0)
      call read_real("PT_G_MIN",ptgmin,20d0)
*      call read_real("PT_G_MAX",ptgmax,14000d0)
      call read_real("PHISOLCUT",isol_phopart,0.7d0)
      call read_real("EFISOLCUT",eff_phopart,1d0)
      call read_real("MLG_MIN",mlAmin,0d0)
      call read_real("MLG_MAX",mlAmax,14000d0)
*      call read_real("MT_MIN",MTmin,0d0)
*      call read_real("MT_MAX",MTmax,14000d0)

c VBF cuts -  only applied to tagging jets
      call read_real("ETAJJ_MIN",etajjmin, 0d0)
      call read_logical("YSIGN",ysign,.false.)
      call read_logical("LRAPIDGAP",lrapidgap,.false.)
      call read_real("DELY_JL",dely_jlep, 0d0)
      call read_logical("GRAPIDGAP",grapidgap,.false.)
      call read_real("DELY_JG",dely_jphot, 0d0)

      call read_real("MDIJ_MIN",mdijmin, 600d0)
      call read_real("MDIJ_MAX",mdijmax,14000d0)

      call read_logical("JVETO",lveto,.false.)
      call read_real("DELY_JVETO",dely_jveto,0d0)
      call read_real("YMAX_VETO",ymax_veto,4.5d0)
      call read_real("PTMIN_VETO",ptmin_veto,10d0)

      call closefile
      min_jets=njet_min !minimal number of jets: set in process.F

      if(ptmin_def(1).lt.ptmin_def(2).or.ptmin_def(2).lt.ptmin_def(3).or.
     1   ptmin_def(3).lt.ptmin_def(4)) then
         print*,"Warning: PT_JET_MIN is not ordered; higher values"
         print*,"should come first"
      endif

c print information on cuts
      print *,""
      print *,"         information on cut parameters"
      print *,"-----------------------------------------------"
      print *,""
      print "(A,4(G9.4e1))"," minimum jet pt        (PT_JET_MIN):    ",ptmin_def
      print *,"maximum jet rapidity                 (Y_JET_MAX): ",ymax_def
      print *,"maximum parton pseudorapidity          (Y_P_MAX): ",ymax_part
      print *,"minimum jet-jet R-separation           (RJJ_MIN): ",rjjmin
      print *,"minimum number of jets                (NJET_MIN): ",njet_min
      print *,"exponent of generalised kT algorithm (PGENKTJET): ",jetgenktp
      print *,"minimum lepton-lepton R-separation     (RLL_MIN): ",rll_min
      print *,"maximum lepton-lepton R-separation     (RLL_MAX): ",rll_max
      print *,"minimum jet-lepton R-separation        (RJL_MIN): ",rjl_min
      print *,"minimum jet-photon R-separation        (RJG_MIN): ",rjg_min
      print *,"minimum lepton-photon R-separation     (RLG_MIN): ",rlg_min   
      print *,"minimum photon-photon R-separation     (RGG_MIN): ",rgg_min
      print *,"maximum photon-photon R-separation     (RGG_MAX): ",rgg_max
      print *,"maximum lepton rapidity                (Y_L_MAX): ",ymax_l
      print *,"minimum lepton pt                     (PT_L_MIN): ",ptlmin
*      print *,"maximum lepton pt                     (PT_L_MAX): ",ptlmax
*      print *,"minimum invisible pt                  (PT_IN_MIN): ",ptinmin
      print *,"minimum invariant dilepton mass        (MLL_MIN): ",mllmin
      print *,"maximum invariant dilepton mass        (MLL_MAX): ",mllmax
      print *,"maximum photon rapidity                (Y_G_MAX): ",ymax_g
      print *,"minimum photon pt                     (PT_G_MIN): ",ptgmin  
*      print *,"maximum photon pt                     (PT_G_MAX): ",ptgmax 
      print *,"photon isolation cut                 (PHISOLCUT): ",isol_phopart
      print *,"efficiency of photon isolation cut   (EFISOLCUT): ",eff_phopart
      print *,"minimum invariant lepton-photon mass   (MLG_MIN): ",mlAmin
      print *,"maximum invariant lepton-photon mass   (MLG_MAX): ",mlAmax
*      print *,"minimum Wgamma transverse mass          (MT_MIN): ",MTmin
*      print *,"maximum Wgamma transverse mass          (MT_MAX): ",MTmax
      print *,"minimum jet rapidity separation      (ETAJJ_MIN): ",etajjmin
      print *,"tagging jets in opposite hemispheres     (YSIGN):   ",ysign
      print *,"leptons fall inside rapidity gap     (LRAPIDGAP):   ",lrapidgap
      print *,"min leptons y-dist from tagging jets   (DELY_JL): ",dely_jlep
      print *,"photons fall inside rapidity gap     (GRAPIDGAP):   ",grapidgap
      print *,"min photons y-dist from tagging jets   (DELY_JG): ",dely_jphot
      print *,"dijet min mass cut on tag jets        (MDIJ_MIN): ",mdijmin
      print *,"dijet max mass cut on tag jets        (MDIJ_MAX): ",mdijmax
      print *,"veto criteria for jets                   (JVETO):   ",lveto
      print *,"minimun veto-tag y-dist             (DELY_JVETO): ",dely_jveto
      print *,"maximum |y| for veto jet             (YMAX_VETO): ",ymax_veto
      print *,"minimum pT for veto jet             (PTMIN_VETO): ",ptmin_veto
      print *,""

c      if (model.eq.2) then
c        print *,""
c        print *,"definition of Lightest Supersymmetric Particle "
c        print *,"-----------------------------------------------"
c        print *,""
c        call loadfile("susy.dat",.true.)
c        call read_Int("LSP",LSP,9999)
c        call closefile
c      else
c        LSP=9999
c      endif
** Lightest susy particle - no effect yet!
      LSP = 9999
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION Cuts(jets, njets, leptons, nl,invisible, ninv, 
     -     photons, nphotons, L)
c*************************************************************************
c     This is the general cut routine. It is intended to be the interface
c     to any basic, specialized or user-defined cut routines. 
c*************************************************************************
c     INPUT 
c     jets : the array with the jet information as it is returned by the 
c            jetdefinition routine.
c     v    : 4-momenta of the particles, which are not jets. 
c     L    : id for momentum set in jets
c
c     OUTPUT
c     Cuts   : .true. or .false. depending on whether the event passes the cuts.
c*************************************************************************

      IMPLICIT NONE
#include "global.inc"
#include "process.inc"

c     input variables
      real*8 partons(0:7,max_jets,max_kin)
      real*8 jets(0:7,max_jets,max_kin) 
      real*8 leptons(0:8,max_v,max_kin), invisible(0:8,max_v,max_kin)
      real*8 photons(0:7,max_v,max_kin)   
      integer npartons(max_kin), ninv(max_kin)
      integer L, njets(max_kin), nl(max_kin), nphotons(max_kin)

c     external functions
      logical Basic_Cuts, VBF_Cuts, JetVeto_Cuts
      external Basic_Cuts, VBF_Cuts, JetVeto_Cuts

      common/partons/partons,npartons


c call of function Basic_Cuts for all processes
      Cuts = Basic_Cuts( partons(0,1,L), npartons(L), jets(0,1,L), 
     1                   njets(L), leptons(0,1,L), nl(L), 
     2                   invisible(0,1,L), ninv(L),
     3                   photons(0,1,L), nphotons(L) )


c call of jet veto for all processes
      if (Cuts) then
         Cuts = JetVeto_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                          nl(L), photons(0,1,L), nphotons(L))
      endif


c call of vbf_cuts only for vbf processes
      SELECT CASE(procID)
      CASE(Hjj, Hjj_AA, AAjj, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW,
     &     Hjj_ZZ_ll, Hjj_ZZ_lnu, Ajj, Zjj_l, Zjj_nu, WPjj, WMjj,
     &     WPWMjj, WPWPjj, WMWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj, HAjj, 
     &     HAjj_AA, HAjj_mu, HAjj_tau, HAjj_bbar, HAjj_WW, 
     &     HAjj_ZZ_ll, HAjj_ZZ_lnu,
     &     GFHjj, GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar,  
     &     GFHjj_WW, GFHjj_ZZ_ll, GFHjj_ZZ_lnu, 
     &     Hjjj, Hjjj_AA, Hjjj_mu,
     &     Hjjj_tau, Hjjj_bbar, Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu)
         if (Cuts) then
            Cuts = VBF_Cuts( jets(0,1,L), njets(L), leptons(0,1,L),
     1                         nl(L), photons(0,1,L), nphotons(L))
         endif
      END SELECT


      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION BASIC_CUTS(partons, npartons, jets, njets,
     1                  leptons, nl, invisible, ninv, photons, nphotons)
c*************************************************************************
c     This routine offers vbf specific cuts, that is cuts for processes 
c     with 2 or more jets and 2 or more leptons.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 partons(0:7,max_jets)
      real*8 jets(0:7,max_jets) 
      real*8 leptons(0:8,max_v)
      real*8 invisible(0:8,max_v)      
      real*8 photons(0:7,max_v)
      integer npartons, njets, nl, nphotons, ninv
      real*8 pt,y,phi
      integer i, j,  k, kk, kmax, mu, nveto, maxnjet
      real*8 ylepmax,ylepmin,m2,mjj2,mll,mlA,yvetomax,yvetomin
      real*8 yphotmax, yphotmin,MT2

      logical pass_cuts

      double precision RSEPS
      external RSEPS
c     photon isolation stuff
      double precision ptsum,factor
      double precision delta(npartons)
      double precision dlist(npartons,nphotons)
      double precision ptlist(npartons,nphotons)
      double precision flist(npartons,nphotons)
      double precision spt(npartons,nphotons)

  
      pass_cuts = .true.
      pass_cuts = pass_cuts .and. (njets.ge.min_jets)

c   JET CUTS
c=================
c...jet-jet r-separation
      do i=1,njets-1
         do j = i+1, njets
            pass_cuts=pass_cuts.and.
     &      ( RSEPS(jets(6,i),jets(7,i),jets(6,j),jets(7,j)).gt.rjjmin)
         enddo
      enddo


c   LEPTON CUTS
c=================
c...maximum lepton rapidity and minimum lepton pt
      do i=1, nl
         pass_cuts=pass_cuts.and. 
     &   ( (abs(leptons(6,i)).lt.ymax_l).and.(leptons(5,i).gt.ptlmin))
*     &   .and.(leptons(5,i).le.ptlmax) )  
      enddo


c...cut on minimum pt of invisible particles
c$$$      do i=1, ninv
c$$$         pass_cuts=pass_cuts.and. 
c$$$     &   ( (invisible(5,i).gt.ptinmin) )
c$$$     
c$$$      enddo

c...lepton-lepton r-separation min and max 
      do i=1, nl-1
         do j = i+1, nl
            pass_cuts=pass_cuts.and.
     -           ( RSEPS(leptons(6,i),leptons(7,i),leptons(6,j),
     -           leptons(7,j)).gt.rll_min .and. RSEPS(leptons(6,i),
     -           leptons(7,i),leptons(6,j),leptons(7,j)).lt.rll_max)
         enddo
      enddo

c... cut on invariant dilepton mass
      do i = 1,nl-1
         do j = i+1,nl
            if(leptons(8,i)*leptons(8,j).lt.0) then !opposite sign leptons only
               mll = mjj2(leptons(0,i),leptons(0,j))
               pass_cuts=pass_cuts.and.(mll.gt.mllmin**2).and.
     -              (mll.lt.mllmax**2)
            endif
         enddo
      enddo

c   PHOTON CUTS
c=================
c...maximum photon rapidity and minimum photon pt
      do i=1, nphotons
         pass_cuts=pass_cuts.and. 
     &   ( (abs(photons(6,i)).lt.ymax_g).and.(photons(5,i).gt.ptgmin))
c...maximum photon pt
*     &   .and.(photons(5,i).le.ptgmax) )     
      enddo

c...photon-photon r-separation min and max
      do i=1, nphotons-1
         do j = i+1, nphotons
            pass_cuts=pass_cuts.and.
     -           ( RSEPS(photons(6,i),photons(7,i),photons(6,j),
     -           photons(7,j)).gt.rgg_min .and. RSEPS(photons(6,i),
     -           photons(7,i),photons(6,j),photons(7,j)).lt.rgg_max)
         enddo
      enddo

c...transverse mass from hep-ph/9305314

c     transverse mass only useful for Wgamma production!!!
c$$$      if(nphotons.eq.1)then
c$$$               MT2 = (sqrt(mjj2(leptons(0,1),photons(0,1))+ 
c$$$     &                (photons(1,1)+leptons(1,1))**2 +(photons(2,1)+leptons(2,1))**2)+
c$$$     &                 invisible(5,1))**2     
c$$$     &                -(photons(1,1)+leptons(1,1)+invisible(1,1))**2
c$$$     &               -(photons(2,1)+leptons(2,1)+invisible(2,1))**2
c$$$!               print*,MT2, mla, MTmin, MTmax 
c$$$               pass_cuts=pass_cuts.and.(MT2.gt.MTmin**2).and.(MT2.le.MTmax**2)
c$$$      endif





! c...NEW photon isolation cut according to Frixione,PLB 429(1998),369
       do i=1, nphotons
         k=0
         do j=1, npartons
            delta(j)=RSEPS(photons(6,i),photons(7,i),partons(6,j),
     -           partons(7,j))
            if (delta(j).lt.isol_phopart) then
               k=k+1
               dlist(k,i)=delta(j)
               ptlist(k,i)=partons(5,j)
               flist(k,i)=(1.-dcos(dlist(k,i)))/(1.-dcos(isol_phopart))
            endif
         enddo
         kmax=k
         do k=1,kmax
            spt(k,i)=0d0
            do kk=1,kmax
               if (dlist(kk,i).le.dlist(k,i)) then
                  spt(k,i)=spt(k,i)+ptlist(kk,i)
               endif
            enddo
            pass_cuts=pass_cuts
     &           .and.(spt(k,i).le.eff_phopart*photons(5,i)*flist(k,i))
         enddo
       enddo


c   ADDITIONAL CUTS
c=================================
c...jet-lepton r-separation
      do i=1, nl
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(leptons(6,i),leptons(7,i),jets(6,j),
     -           jets(7,j)).gt.rjl_min )
         enddo
      enddo

c...jet-photon r-separation
      do i=1, nphotons
         do j=1, njets
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(photons(6,i),photons(7,i),jets(6,j),
     -           jets(7,j)).gt.rjg_min )
         enddo
      enddo

c...lepton-photon r-separation
      do i=1, nphotons
         do j=1, nl
            pass_cuts=pass_cuts.and.
     &           ( RSEPS(photons(6,i),photons(7,i),leptons(6,j),
     -           leptons(7,j)).gt.rlg_min )
         enddo
      enddo

c...minimal invariant lepton+photon mass for all combinations of charged leptons and photons

      do i = 1,nl
         do j = 1,nphotons
               mlA = mjj2(leptons(0,i),photons(0,j))
               pass_cuts=pass_cuts.and.(mlA.gt.mlAmin**2).and.
     &             (mlA.le.mlAmax**2)
         enddo
      enddo


      
      BASIC_CUTS = pass_cuts
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION VBF_Cuts(jets, njets, leptons, nl, photons, 
     -     nphotons)
c*************************************************************************
c     This routine offers vbf specific cuts, that is cuts for processes 
c     with 2 or more jets and 2 or more leptons.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "cuts.inc"

c     input variables
      real*8 jets(0:7,max_jets) 
      integer njets, nl, nphotons
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      real*8 pt,y,phi
      integer i, j, mu, nveto
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      logical pass_cuts

      double precision RSEPS
      external RSEPS

      logical Basic_Cuts
      external Basic_cuts

      
      pass_cuts = .true.

c...jet-jet rapidity separation
      if (njets.ge.2) then
        pass_cuts=pass_cuts.and.
     &        (abs(jets(6,1)-jets(6,2)).gt.etajjmin)
      endif

c...jets #1 and #2 must have opposite sign rapidity
      if (ysign) then
         if (njets.ge.2) then
            pass_cuts=pass_cuts.and. ( (jets(6,1)*jets(6,2)).le.0d0 )
         endif
      endif

c...leptons fall inside rapidity gap
      if(lrapidgap) then
         if (njets.ge.2) then
            ylepmax = max(jets(6,1),jets(6,2)) - dely_jlep
            ylepmin = min(jets(6,1),jets(6,2)) + dely_jlep
            do i=1,nl
               pass_cuts=pass_cuts.and.(leptons(6,i).lt.ylepmax.and.
     &              leptons(6,i).gt.ylepmin)      
            enddo
         endif
      endif

c...photons fall inside rapidity gap
      if(grapidgap) then
         if (njets.ge.2) then
            yphotmax = max(jets(6,1),jets(6,2)) - dely_jphot
            yphotmin = min(jets(6,1),jets(6,2)) + dely_jphot
            do i=1,nphotons
               pass_cuts=pass_cuts.and.(photons(6,i).lt.yphotmax.and.
     &              photons(6,i).gt.yphotmin)      
            enddo
         endif
      endif

c...  tagging jets invariant dijet mass cut
      if (njets.ge.2) then
         m2=mjj2(jets(0,1),jets(0,2))
         pass_cuts=pass_cuts.and.(m2.ge.mdijmin**2.and.m2.le.mdijmax**2)
      endif

      VBF_cuts = pass_cuts
      
      RETURN
      END

c*************************************************************************
      LOGICAL FUNCTION JetVeto_Cuts(jets, njets, leptons, nl, photons, 
     -     nphotons)
c*************************************************************************
c     This routine offers jet veto cuts, that is cuts on additional jets
c     apart from the standard number of the LO process.
c*************************************************************************
      IMPLICIT NONE
#include "global.inc"
#include "process.inc"
#include "cuts.inc"

c     input variables
      real*8 jets(0:7,max_jets) 
      integer njets, nl, nphotons
      real*8 leptons(0:8,max_v)
      real*8 photons(0:7,max_v)
      real*8 pt,y,phi
      integer i, j, mu, nveto, maxnjet
      real*8 ylepmax,ylepmin,m2,mjj2,mll,yvetomax,yvetomin
      real*8 yphotmax, yphotmin

      logical pass_cuts
      
      pass_cuts = .true.

C... veto jet cut      
      nveto=0
      if(lveto) then
         SELECT CASE(procID)
c vbf processes have maximum two jets 
         CASE(Hjj, Hjj_AA, Hjj_mu, Hjj_tau, Hjj_bbar, Hjj_WW,
     &        Hjj_ZZ_ll, Hjj_ZZ_lnu, Ajj, Zjj_l, Zjj_nu, WPjj, WMjj,
     &        WPWMjj, WPWPjj, WMWMjj, ZZjj_ll, ZZjj_lnu, WPZjj, WMZjj, HAjj, 
     &        HAjj_AA, HAjj_mu, HAjj_tau, HAjj_bbar, HAjj_WW, 
     &        HAjj_ZZ_ll, HAjj_ZZ_lnu,
     &        GFHjj, GFHjj_AA, GFHjj_mu, GFHjj_tau, GFHjj_bbar,
     &        GFHjj_WW, GFHjj_ZZ_ll, GFHjj_ZZ_lnu, 
     &        Hjjj, Hjjj_AA, Hjjj_mu,
     &        Hjjj_tau, Hjjj_bbar, Hjjj_WW, Hjjj_ZZ_ll, Hjjj_ZZ_lnu)
            yvetomax = max(jets(6,1),jets(6,2)) - dely_jveto
            yvetomin = min(jets(6,1),jets(6,2)) + dely_jveto
            maxnjet = 2
         CASE DEFAULT
c for all other processes just veto the additional jet
            yvetomax =  100d0
            yvetomin = -100d0
            maxnjet = njet_min
         END SELECT

         if(njets.gt.njet_min)then
            do i=maxnjet+1,njets
               if(jets(6,i).le.yvetomax.and.
     &            jets(6,i).ge.yvetomin.and.
     &            abs(jets(6,i)).le.ymax_veto.and.
     &            jets(5,i).gt.ptmin_veto
     &           ) nveto=nveto+1            
            enddo
           pass_cuts=pass_cuts.and.(nveto.eq.0)
         endif
      endif
      
      
      JetVeto_cuts = pass_cuts
      
      RETURN
      END


c*************************************************************************
      SUBROUTINE DefineLeptons(v, Lmax, leptons, n_leptons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the leptons out of "v" into "leptons". Only e, mu, tau particles are
c     taken. In addition pt, y and phi information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 leptons(0:8,max_v,max_kin)
      integer n_leptons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,8
               leptons(mu,i,l) = 0d0
            enddo
         enddo

         n_leptons(l)=0
c     get leptons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.11) ! check if it is a lepton
     &           .or.(abs(particle_IDs(i)).eq.13)
     &           .or.(abs(particle_IDS(i)).eq.15) ! taus!!
     &           ) then
               n_leptons(l)=n_leptons(l)+1
               do mu = 0,3      ! transfer to "leptons"
                  leptons(mu,n_leptons(l),l) = v(mu,i,l)
               enddo
               leptons(8,n_leptons(l),l) = particle_IDs(i)
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_leptons(l)
            call lego( leptons(0,i,l), pt, y, phi )
            leptons(4,i,l) = 0.0 ! mass  
            leptons(5,i,l) = pt  ! transverse momentum  
            leptons(6,i,l) = y   ! rapidity 
            leptons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     

c*************************************************************************
      SUBROUTINE DefinePhotons(v, Lmax, photons, n_photons)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the photons out of "v" into "photons". In addition pt, y and phi 
c     information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 photons(0:7,max_v,max_kin)
      integer n_photons(max_kin), Lmax
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax

         do i=1, max_v
            do mu=0,7
               photons(mu,i,l) = 0d0
            enddo
         enddo

         n_photons(l)=0
c     get photons out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.22) ! check if it is a photon
     &           ) then
               n_photons(l)=n_photons(l)+1
               do mu = 0,3      ! transfer to "photons"
                  photons(mu,n_photons(l),l) = v(mu,i,l)
               enddo
            endif 
         enddo
c     calculate rapidity, pt, phi-angle for the leptons
         do i=1, n_photons(l)
            call lego(photons(0,i,l), pt, y, phi )
            photons(4,i,l) = 0.0 ! mass  
            photons(5,i,l) = pt ! transverse momentum  
            photons(6,i,l) = y  ! rapidity 
            photons(7,i,l) = phi ! azimuthal angle
         end do
         
      enddo

      RETURN
      END
     
c*************************************************************************
      SUBROUTINE DefineInvisible(v, Lmax, invisible, n_invisible)
c*************************************************************************
c     Similar to "DefineJets" in jetdefinition.f this routine extracts
c     the invisible particles out of "v" into "invisible". In addition pt,
c     y and phi information is calculated and stored.
c*************************************************************************
      implicit none
#include "global.inc"
      real*8 v(0:3,max_v,max_kin)
      real*8 invisible(0:8,max_v,max_kin)
      integer n_invisible(max_kin), Lmax
      integer LSP
      common /LSP_def/ LSP
      real*8 pt,y,phi
      integer i, l, mu

      do l=1, Lmax
         do i=1, max_v
            do mu=0,8
               invisible(mu,i,l) = 0d0
            enddo
         enddo

         n_invisible(l)=0
c     get invisible particles out of decay_particles     
         do i=1, n_v
            if ( (abs(particle_IDs(i)).eq.12) ! check if it is a neutrino
     &           .or.(abs(particle_IDs(i)).eq.14)
     &           .or.(abs(particle_IDS(i)).eq.16)
     &           .or.(abs(particle_IDS(i)).eq.LSP)) then
               n_invisible(l)=n_invisible(l)+1
               do mu = 0,3      ! transfer to "invisible"
                  invisible(mu,n_invisible(l),l) = v(mu,i,l)
               enddo
               invisible(8,n_invisible(l),l) = particle_IDs(i)
            endif 
         enddo
	 

c     calculate rapidity, pt, phi-angle for the invisible particles
         do i=1, n_invisible(l)
            call lego( invisible(0,i,l), pt, y, phi )
            invisible(4,i,l) = 0.0 ! mass  
            invisible(5,i,l) = pt  ! transverse momentum  
            invisible(6,i,l) = y   ! rapidity 
            invisible(7,i,l) = phi ! azimuthal angle
         end do
      enddo    
      

      RETURN
      END
 
